module Rack
  module OAuth2
    module Server
      class Authorize
        module Extension
          # Define your extension in this namespace and load it explicitly.
          # extension/code_and_token.rb would be good example for you.
        end
      end
    end
  end
end